OneLoop::resv = "ieps,jg,jgm,li2,lim,lip,x,y,z,$q are reserved symbols.";
OneLoop::ldef = "The lists 'pars', 'momext', and/or 'rels' are undefined.";
SubSoft::nosoft = "Cannot identify soft singularity in the limit `1`.";
SubSoft::toosg = "Too many singular propagators in the limit `1`.";
SubColl::nocoll = "Cannot identify collinear singularity in the limit `1`.";
WriteCode::nden = "Two different denominator terms found.";
WriteCode::symb = "Expression contains unidentified symbol(s) `1`.";


ReSet[] := ($ol$subt = {});

ReSet[];

QSort[exprin_] := Sort[exprin, #2=!=$q && (#1===$q || OrderedQ[{#1,#2}]) &];
TSort[exprin_] := Sort[exprin, #2=!=$q.$q && (#1===$q.$q || OrderedQ[{#1,#2}]) &];

DotSimplify[exprin_] := Module[{eold, expr=exprin},
  While[eold =!= expr,
    eold = expr;
    expr = expr /. Dot[arga_,argb_] :> Distribute[Dot[arga,argb]];
    expr = expr //. 
      {Dot[fac_*arga_,argb_] :> fac*Dot[arga,argb] /; FreeQ[fac,$ol$moms],
       Dot[arga_,fac_*argb_] :> fac*Dot[arga,argb] /; FreeQ[fac,$ol$moms]}
  ];
  expr = expr /. expd_Dot :> QSort[expd];
  Return[expr];
];

mysimp2[exprin_] := Module[{etmp = Together[exprin]},
  Collect[Numerator[etmp], Join[{Dot[_,_]},pars,momext], Simplify] /
  	   Simplify[Denominator[etmp]]
];

AddSub[expr_, abbr_] := ($ol$lsubs = Append[$ol$lsubs, abbr -> expr]);


Options[OneLoop] = {SimpLevel -> 3, 
                    NestedExpressions -> False,
		    FPCountStart -> 1};

OneLoop[aden_, anum_, q_, opts___Rule] := 
Module[{simplev,neste,varc,num,den,exp,sum,trn,jac,n,j,k,dens,int,nvar,vars,
        varr,abbc},
  simplev = SimpLevel /. {opts} /. Options[OneLoop];
  neste = NestedExpressions /. {opts} /. Options[OneLoop];
  varc = FPCountStart /. {opts} /. Options[OneLoop];
  If[!FreeQ[{aden,anum,q,varc}, ieps|jg|jgm|li2|lim|lip|x|y|z|$q],
    Message[OneLoop::resv]; Return[]];
  If[!ValueQ[pars] || !ValueQ[momext] || !ValueQ[rels],
    Message[OneLoop::ldef]; Return[]];
  $ol$moms = Alternatives @@ Join[momext, {$q}];
  $ol$lsubs = {};
  num = anum /. q->$q;
  den = Sort[Union[aden], OrderedQ[{#2,#1} /. Dot[_,_]->0]&];
  exp = Map[Count[aden, #]&, den];	(* power of each denominator term *)
  n = 0;
  (* introduce Feynman parameters: *)
  den = Plus @@ Map[z[++n]*# &, den /. q->$q];
  sum = Plus @@ Table[z[j], {j,n}];
  (* variable transformation onto hyper-cube: *)
  trn = Table[z[j] -> Product[y[k], {k,j}]*If[j===n, 1, 1-y[j+1]], {j,n}];
  jac = Det[Outer[D, Map[Last,trn], Table[y[j],{j,n}]]];
  trn = trn /. y[1]->1;
  jac = jac /. y[1]->1;
  den = Collect[DotSimplify[den], Dot[_,_], Simplify[#, sum==1]&];
  (* shift loop mom. to eliminate linear term: *)
  If[simplev > 0,
    num = Collect[DotSimplify[num] /. rels, Dot[_,_], Simplify]
  ];
  num = num /. $q -> $q - Coefficient[den /. Dot->Times, $q, 1]/2;
  den = den /. $q -> $q - Coefficient[den /. Dot->Times, $q, 1]/2;
  den = Collect[DotSimplify[den], Dot[_,_], Simplify[#, sum==1]&];
  If[neste,
    AddSub[Simplify[den /. rels /. $q.$q->0], abbv[1]];
    den = $q.$q + abbv[1]
  ];
  dens = 1/den^(Plus @@ exp);
  dens *= Gamma[Sum[exp[[j]], {j,n}]] * 
	  (Product[z[j]^(exp[[j]]-1)/Gamma[exp[[j]]], {j,n}]) /. sum->1;
  (* make sure we have always the same sign convention: *)
  dens = dens /. (A_ - $q.$q)^expd_ :> (-1)^expd * ($q.$q - A)^expd;
  num *= Numerator[dens];
  dens = 1/Denominator[dens] /. rels;
  num = DotSimplify[num] /. rels;
  (* collect all momentum dot product in numerator that involve the loop
     momentum and give them the header T: *)
  num = Expand[num, Dot[_,__]];
  num = num /. Power[Dot[arga_,argb_], nA_:1] :> 
		  (T @@ Table[Dot[arga,argb], {nA}]) /; 
		  !FreeQ[Dot[arga,argb], $q] //.
               T[arga__] * T[argb__] :> T[arga,argb];
  num = num /. expd_Dot :> QSort[expd] /. expt_T :> TSort[expt];
  num = num /. expt_T :> 0 /; OddQ[Length[Position[expt, $q]]];
  (* tensors of odd rank are zero;
     tensors of even rank evaluate to combinations of metric tensors: *)
  num = num  //. 
       T[argt___, $q.arga_, $q.argb_, $q.argc_, $q.argd_, $q.arge_, $q.argf_] :>
              1/($d*(2+$d)*(4+$d)) * T[argt, $q.$q, $q.$q, $q.$q] * 
	        (arga.argb*argc.argd*arge.argf + arga.argc*argb.argd*arge.argf +
		 arga.argd*argb.argc*arge.argf + arga.argb*argc.arge*argd.argf +
		 arga.argc*argb.arge*argd.argf + arga.arge*argb.argc*argd.argf +
		 arga.argb*argc.argf*argd.arge + arga.argc*argb.argf*argd.arge +
		 arga.argf*argb.argc*argd.arge + arga.argd*argb.arge*argc.argf +
		 arga.arge*argb.argd*argc.argf + arga.argd*argb.argf*argc.arge +
		 arga.argf*argb.argd*argc.arge + arga.arge*argb.argf*argc.argd +
		 arga.argf*argb.arge*argc.argd)  /;
	     	FreeQ[{argt}, $q.(mom_/;mom=!=$q)] && 
		FreeQ[{arga,argb,argc,argd,arge,argf}, $q]  //.
	     T[argt___, $q.arga_, $q.argb_, $q.argc_, $q.argd_] :>
              1/($d*(2+$d)) * T[argt, $q.$q, $q.$q] * (arga.argb*argc.argd +
	                   arga.argc*argb.argd + arga.argd*argb.argc)  /;
	     	FreeQ[{argt}, $q.(mom_/;mom=!=$q)] && 
		FreeQ[{arga,argb,argc,argd}, $q]  //.
	     T[argt___, $q.arga_, $q.argb_] :> 1/$d*T[argt, $q.$q] * arga.argb /;
	     	  FreeQ[{argt}, $q.(mom_/;mom=!=$q)] && 
		  FreeQ[{arga,argb}, $q];
  If[simplev > 2,
    num = Collect[num, T[__], Simplify[# //. rels, sum==1]&],
    num = Collect[num, T[__]] //. rels
  ];
  If[neste,
    abbc = 1;
    num = Collect[num, T[__], myhold];
    num = num /. myhold[arg_] :> Collect[Numerator[arg], $d, 
    			(++abbc; AddSub[#, abbv[abbc]]; abbv[abbc]) &]/
		     Denominator[arg];
  ];
  int = num * dens;
  (* cancel (loop mom.)^2 in numerator against denominator: *)
  int = int /. expt:T[($q.$q)..] :> $q^(2*Length[expt]) /. Dot[$q,$q]->$q^2;
  int = int //. ((R1_:1)*$q^na_ + (R2_:0)) * ($q^2 + A_)^nA_ :> 
          R1*($q^2 + A)^(nA+na/2) + 
	  Collect[R2 - R1*(($q^2+A)^(na/2) - $q^na), {$q,Dot[_,_]}]*
	    ($q^2 + A)^nA /; FreeQ[R1, $q];
  If[simplev > 0,
    int = Collect[int, ($q^2 + _)^_, mysimp2],
    int = Collect[int, ($q^2 + _)^_]
  ];
  (* evaluate the loop mom. integration: *)
  int = int /. ($q^2 + A_)^nA_ :> (-1)^(-nA) * Gamma[-nA-$d/2]/Gamma[-nA] *
				   (-Simplify[A /. $ol$lsubs]-ieps)^($d/2+nA);
  (* perform variable transformation onto hyper-cube: *)
  int = (int /. trn /. z[nn_] :> y[nn]) * jac;
  $ol$lsubs = ($ol$lsubs /. trn /. z[nn_] :> y[nn]);
  If[simplev > 1, 
    int = Collect[int, (_-ieps)^_, mysimp2] /. (A_-ieps) :> Simplify[A-ieps];
  ];  
  (* rename Feynman parameter variables: *)
  vars = Union[Join[Extract[int, Position[int, y[_]]],
  		    Extract[$ol$lsubs, Position[$ol$lsubs, y[_]]]]];
  nvar = Length[vars];
  varr = Table[vars[[j]] -> x[j+varc-1], {j,nvar}];
  int = int /. varr;
  $ol$lsubs = $ol$lsubs /. varr;
  Append[$ol$lsubs, $ol$res->int]
];


SubSoft[aden_, anum_, q_, mom0_] :=
Module[{qrepl,softsubfac1,softsubfac2,sing,sing2,i,m,m1,m2,moml,s},
  qrepl = First[Solve[mom0==0, q]];
  $ol$moms = Alternatives @@ Join[momext, {q}];
  softsubfac1 = DotSimplify[anum /. qrepl /. 
  			    {Dot[0,_]->0, Dot[_,0]->0}] /. rels;
  (* find propagators that vanish in soft limit: *)
  sing = Select[aden,
             Simplify[DotSimplify[# /. qrepl] /. rels /. Dot[0,0]->0] === 0 &];
  If[Length[sing] > 3,
    Message[SubSoft::toosg, mom0->0]; Return[anum]
  ];
  (* identify the two massive singular propagators: *)
  sing2 = Select[sing, FreeQ[# /. qrepl, Dot[0,0]]&];
  If[Length[sing2] =!= 2 || Length[sing] =!= 3,
    Message[SubSoft::nosoft, mom0->0]; Return[anum]
  ];
  {m1,m2} = -sing2 /. Dot[_,_]->0;
  moml = Map[Extract[#, Position[#, Dot[_,_]]][[1,1]] &, sing2];
  (* select remaining (non-singular) propagators: *)
  softsubfac2 = Times @@ Complement[aden, sing];
  (* store the integrated subtraction terms in $ol$subt: *)
  s = DotSimplify[Dot[moml[[1]]-moml[[2]], moml[[1]]-moml[[2]]]] /. rels;
  If[m1 === 0 && m2 === 0,
    $ol$subt = Join[$ol$subt,
      {$ol$subr -> 1/s*(1/$e^2 + 1/$e*(-lim[-s]) + (lim[-s]^2/2 - Pi^2/12) +
	                $e*(Pi^2/12*lim[-s] - lim[-s]^3/6 - 2.80479944070572)) *
          Simplify[DotSimplify[softsubfac1/(softsubfac2 /. qrepl)] /. rels]}],
  (* else *)
    If[m1 =!= 0 && m2 =!= 0,
      i = 1+ Count[$ol$subt, $ol$subr -> _];
      $ol$subt = Join[$ol$subt,
        {xs[i] -> Evaluate[Simplify[Sqrt[1+(m1-m2)^2/s^2 - 2*(m1+m2)/s]]],
         xa[i] -> (1 + (m2-m1)/s - xs[i])/2,
         xb[i] -> (1 + (m2-m1)/s + xs[i])/2,
         $ol$subr -> 1/(2*s*xs[i])*(1/$e*(lip[-xa[i]] - lim[-xb[i]] - 
	                                  lip[1-xa[i]] + lim[1-xb[i]]) +
                        (lim[-xb[i]]^2/2 - lip[-xa[i]]^2/2 + 
	                 lim[-xb[i]]*(lip[s] - lip[-xa[i]]) + 
			 lip[1-xa[i]]^2/2 - lim[1-xb[i]]^2/2 +
                         lip[s]*(lip[1-xa[i]] - lim[1-xb[i]]) +
                         lip[1-xa[i]]*(lim[1-xb[i]] - 2*lip[(xb[i]-1)/xs[i]]) -
                         lip[-xa[i]]*(lip[s] - 2*Log[xb[i]/xs[i]]) -
                         2*li2[(1-xa[i])/xs[i]] + 2*li2[-xa[i]/xs[i]]) -
                     $e*(lim[-xb[i]]^3/6 - lip[-xa[i]]^3/6 + lip[1-xa[i]]^3/6 +
                         lim[-xb[i]]^2*(lip[s] - lip[-xa[i]] + 2*Log[xs[i]])/2 -
                         lim[1-xb[i]]*(Pi^2/12 + lip[s]^2/2 + (lip[s]/2 -
			               Log[(1-xa[i])/xs[i]])*lim[1-xb[i]] +
				       lim[1-xb[i]]^2/6) +
			 lip[1-xa[i]]*(Pi^2/12 + lip[s]^2/2 - lim[1-xb[i]]^2/2 +
			               (lim[1-xb[i]] - 2*lip[(xb[i]-1)/xs[i]])*
				       lip[s]) +
			 lip[1-xa[i]]^2*(lip[s]/2 + lim[1-xb[i]]/2 - 
			                 lip[(xb[i]-1)/xs[i]]) -
                         lip[-xa[i]]^2*(lip[s]/2 - Log[xb[i]/xs[i]]) -
			 2*(lip[s] + lip[1-xa[i]])*li2[(1-xa[i])/xs[i]] +
			 2*(lip[s] + lip[-xa[i]])*li2[-xa[i]/xs[i]] -
			 lip[-xa[i]]*(Pi^2/12 + lip[s]^2/2 - 
			              2*lip[s]*Log[xb[i]/xs[i]]) +
			 2*lim[1-xb[i]]*li2[(xb[i]-1)/xs[i]] +
			 lim[-xb[i]]*(Pi^2/12 - lip[-xa[i]]^2/2 + lip[s]^2/2 -
			              lip[-xa[i]]*lip[s] - 2*li2[xb[i]/xs[i]]) +
			 2*(li3[(1-xa[i])/xs[i]] - li3[-xa[i]/xs[i]] -
			    li3[(xb[i]-1)/xs[i]] + li3[xb[i]/xs[i]]))) *
          Simplify[DotSimplify[softsubfac1/(softsubfac2 /. qrepl)] /. rels]}],
    (* else *)
      m = m1+m2;
      $ol$subt = Join[$ol$subt,
         {$ol$subr -> 1/(2*(s-m))*(1/$e^2 + 1/$e*(Log[m] - 2*lim[m-s]) +
                        (Pi^2/12 - Log[m]^2/2 + lim[m-s]^2 - 2*li2[-s/(m-s)]) +
	             $e*((Log[m]-2*lim[m-s])^3/6 - Pi^2/6*lim[m-s] -
                         Pi^2/4*Log[m] + lim[-s]*(Log[m]-lim[m-s])^2 +
                         2*Log[m]*li2[-s/(m-s)] +
                         2*li3[m/(m-s)] - 2*li3[-s/(m-s)] - 2.80479944070572)) *
          Simplify[DotSimplify[softsubfac1/(softsubfac2 /. qrepl)] /. rels]}]
    ]
  ];
  (* return the subtracted integrand numerator: *)
  Return[anum - softsubfac1*softsubfac2/(softsubfac2 /. qrepl)]
];


SubColl[aden_, anum_, q_, momru_] :=
Module[{nosing},
  (* find propagators that do not vanish in collinear limit: *)
  nosing = Select[aden,
            (DotSimplify[# /. momru] /. rels /. Dot[0,0]->0) =!= 0 &];
  If[Length[aden]-Length[nosing] =!= 2,
    Message[SubColl::nocoll, mom0->0]; Return[anum]
  ];
  (* return the subtracted integrand numerator: *)
  Return[anum - ((anum /. momru) * 
  		 (Times @@ Map[# / (# /. momru) &, nosing]) /. Dot[0,0]->0)]
];


Options[WriteCode] = {SimpFunc -> Simplify,
                      EpsRange -> {0,-Infinity},
		      Deform -> False,
		      WorkingDirectory -> "."};

WriteCode[exprin_, opts___Rule] := 
Module[{simpfunc,range,deform,wdir,vars,nvar,varr,expr2,ints,ana1,anas,alist,
        dlist,analist,olist,maxr,maxra,minr,i,j,G,Gx,Grem,DG,jacG,jacGmat,
	jacGlabel,parss,f},
  simpfunc = SimpFunc /. {opts} /. Options[WriteCode];
  range = EpsRange /. {opts} /. Options[WriteCode];
  deform = Deform /. {opts} /. Options[WriteCode];
  wdir = WorkingDirectory /. {opts} /. Options[WriteCode];
  (* if the input is a list of expressions, work on each item in the list
     one by one, and put the output of each item into a subdirectory: *)
  If[Union[Map[Head,exprin]] === {List},
    i = ToCharacterCode["a"];
    Table[(WriteCode[exprin[[j]], WorkingDirectory -> 
             wdir <> "/" <> FromCharacterCode[i+j-1], opts];
	   ReSet[]),
	  {j, Length[exprin]}];
    MakeMultScript[wdir, Table[wdir <> "/" <> FromCharacterCode[i+j-1],
	                       {j, Length[exprin]}], 
	           If[deform, Length[pars]+1, Length[pars]]];
    Return[]
  ];
  (*Print[ByteCount[exprin], " ", wdir];*)
  vars = Union[Extract[exprin, Position[exprin, x[_]]]];
  nvar = Length[vars];
  varr = Table[vars[[j]] -> z[j], {j,nvar}];
  expr2 = exprin /. varr /. z->x;
  (* find denominator of Feynman parameter integral: *)
  Gx = Union[Extract[expr2, Position[expr2, _-ieps]]] /. ieps->0;
  If[Length[Gx] > 1, Message[WriteCode::nden]; Return[], Gx = Gx[[1]]];
  (* factor out any overall powers of the Feynman parameters
     since they do not change sign: *)
  G = Simplify[DeleteCases[Factor[Gx], x[_]^(nA_:1)]];
  Grem = Times @@ Cases[Factor[Gx], x[_]^(nA_:1)];
  (* check for undeclared symbols: *)
  If[(alist = Complement[DeleteCases[
  		  Variables[Map[Last,expr2] /. ieps->0 /. $d->7/2 /. $e->1/4],
  		x[_]|abbv[_]|xee[_]|xe[_]|xf[_]|xd[_]], pars]) =!= {},
    Message[WriteCode::symb, alist]; Return[]
  ];
  (* expand in d-4: *)
  ints = Series[$ol$res /. expr2 /. ieps->0 /. Gx^nG_ :> G^nG*Grem^nG /. 
  		$d->4-2*$e, {$e,0,0}] /. EulerGamma->0;
  For[i = 1, i <= Length[ints[[3]]], i++,
    ints[[3,i]] = Collect[ints[[3,i]], Log[_], simpfunc];
  ];
  (* collect the analytical formulas of the subtraction terms: *)
  anas = Plus @@ Map[$ol$subr /. # /. $ol$subr->0 &, $ol$subt];
  anas = Series[Collect[anas, $e] /. $d->4-2*$e, {$e,0,0}] /. EulerGamma->0;
  (* determine highest 1/epsilon pole: *)
  maxr = -Max[ints[[4]], range[[2]]];
  maxra = -Max[Min[ints[[4]], If[anas===0, 0, anas[[4]]]], range[[2]]];
  minr = -Min[0, range[[1]]];
  (* try to see if leading pole can be integrated analytically (if it is just
     a polynomial): *)
  If[FreeQ[ints, abbv],
    ana1 = SeriesCoefficient[ints,-maxr];
    If[FreeQ[ana1, Log[_]] && FreeQ[Denominator[Together[ana1]], x[_]],
      vars = Table[{x[i],0,1}, {i,nvar}];
      anas += Series[Integrate @@ Prepend[vars, ana1]/$e^maxr, {$e,0,0}];
      maxr--
    ]
  ];
  (* list of all abbreviations: *)
  alist = Select[expr2, FreeQ[#,$ol$res]&];
  (* perform contour deformation: *)
  If[deform,
    ints = ints /. x->z;
    alist = alist /. x->z;
    DG = Table[Simplify[D[G,x[i]]], {i,nvar}];
    dlist = Table[x[i] - I*lam*x[i]*(1-x[i])*DG[[i]], {i,nvar}];
    alist = Join[Table[z[i] -> dlist[[i]], {i,nvar}], alist];
    jacGmat = Simplify[Outer[D, dlist, Table[x[i], {i,nvar}]]];
    jacGlabel = Table[jgm[i,j], {i,nvar}, {j,nvar}];
    jacG = Det[jacGlabel];
    alist = Join[alist, 
    	  Map[Rule @@ # &, Transpose[{Flatten[jacGlabel], Flatten[jacGmat]}]]];
    alist = Append[alist, jg -> jacG];
    jacG = jg;
    parss = Append[pars, lam],
  (* else *)    
    jacG = 1;
    parss = pars
  ];
  (* add final result (for each power of epsilon) to list of assignments: *)
  alist = Join[alist, 
     Table["res"[i+1] -> jacG*SeriesCoefficient[ints,-minr-i], {i,0,maxr-minr}]];
  (* list of analytical expressions: *)
  analist = Join[Select[$ol$subt, FreeQ[#,$ol$subr]&],
     Table["ana"[i+1] -> SeriesCoefficient[anas,-minr-i], {i,0,maxra-minr}] /.
     	SeriesCoefficient[0,_]->0];
  (* generate Fortran output: *)
  If[wdir =!= "." && FileNames[wdir] === {},
    CreateDirectory[wdir]
  ];
  f = OpenWrite[wdir <> "/fdef.f"];
  WriteString[f, "#define NVAR " <> ToString[nvar] <> "\n"];
  WriteString[f, "#define NCOMP " <> ToString[maxr-minr+1] <> "\n"];
  WriteString[f, "#define NCOMPA " <> ToString[maxra-minr+1] <> "\n"];
  Close[f];
  f = OpenWrite[wdir <> "/fvar.f"];
  i = 1;
  spars = "";
  While[i <= Length[parss],
    For[j = Length[parss], True, j--,
      spart = StringTake[ToString[Take[parss, {i,j}]], {2,-2}];
      If[spars =!= "" && StringLength[spart] <= 63, Break[]];
      If[StringLength[spart] <= 48, Break[]];
    ];
    spars = spars <> StringReplace[spart, {"["->"(", "]"->")"}];
    spars = spars <> ",\n     &  ";
    i = j+1;
  ];
  WriteString[f, "      double precision " <> spars(* <> "\n"*)];
  WriteString[f, "xs, xa, xb\n      double complex xee, xe, xf, xd\n"];
  WriteString[f, "      common/var/ " <> spars(* <> "\n"*)];
  WriteString[f, "xs(10), xa(10), xb(10), xee(10), xe(10), xf(10), xd(10)\n"];
  Close[f];
  f = OpenWrite[wdir <> "/fvar2.f"];
  WriteString[f, "      if(iargc() < " <> ToString[Length[parss]] <> ") then\n"];
  WriteString[f, "        write(0,*) \"Too few parameters!\"\n"];
  WriteString[f, "        stop\n      endif\n"];
  For[i = 1, i <= Length[parss], i++,
    WriteString[f, "      call getarg(" <> ToString[i] <> ",carg)\n"];
    WriteString[f, "      read(carg,*) " <> ToString[parss[[i]]] <> "\n"];
  ];
  Close[f];
  Put[FortranAssign[Evaluate[Map[First, alist]], Evaluate[Map[Last, alist]], 
        AssignOptimize->False, AssignToArray->{x,y,z,jgm,abbv},
	AssignReplace->{"\""->""}], wdir <> "/fint.f"];
  Put[FortranAssign[Evaluate[Map[First, analist]], Evaluate[Map[Last, analist]], 
        AssignOptimize->False, AssignToArray->{xs,xa,xb,xee,xe,xf,xd}, 
	AssignReplace->{"\""->""}], wdir <> "/fana.f"];
  If[FileNames[wdir <> "/fwrap1.F"] === {},
    CopyFile[ToFileName[$NicoDir, "fwrap1.F"], wdir <> "/fwrap1.F"]
  ];
  If[FileNames[wdir <> "/runpar.sys"] === {},
    CopyFile[ToFileName[$NicoDir, "runpar.sys"], wdir <> "/runpar.sys"]
  ];
  (* compile Fortran output: *)  
  Run[$Compiler <> " -o " <> wdir <> "/run1 " <> wdir <> 
      "/fwrap1.F " <> ToFileName[$Cuba, "libcuba.a"]];
];

(* produce a perl script to orchestrate the numerical run, for the case that
   there are several pieces in separate subdirectories: *)
MakeMultScript[wdir_String, subdirs_List, npars_Integer] := Module[{f},
  Run["cp " <> First[subdirs] <> "/runpar.sys " <> wdir <> "/runpar.sys"];
  Scan[DeleteFile[# <> "/runpar.sys"]&, subdirs];
  Scan[Run["ln -s ../runpar.sys " <> # <> "/runpar.sys"]&, subdirs];
  f = OpenWrite[wdir <> "/run1"];
  WriteString[f, "#!/usr/bin/perl -w\n\nif($#ARGV < " <> ToString[npars-1] <>
                 ") {\n    print \"Too few parameters!\\n\";\n    exit 1;\n}\n\n"];
  WriteString[f, "@totr = ( 0, 0, 0 );\n@toti = ( 0, 0, 0 );\n" <>
                 "@terr = ( 0, 0, 0 );\n@teri = ( 0, 0, 0 );\n" <>
		 "$maxd = 1;\n\n"];
  Scan[WriteString[f,
    "system \"" <> # <> "/run1 @ARGV >" <> wdir <> "/out\";\n" <>
    "if($? != 0) {\n    print \"Error running " <> # <> 
    "/run1! Exit code $?.\\n\";\n    exit 1;\n}\n" <>
    "open($in,  \"<\",  \"" <> wdir <> "/out\")  or die \"Can't open " <> wdir <> "/out: $!\";\n" <>
    "$i = 0;\nwhile (<$in>) {\n" <>
    "    ($resr,$resi,$errr,$erri) = ($_ =~ /\(([0-9\\.\-E]+),([0-9\\.\-E]+)\) " <>
    "\(([0-9\\.\-E]+),([0-9\\.\-E]+)\)/);\n" <>
    "    $totr[$i] += $resr;\n    $toti[$i] += $resi;\n" <>
    "    $terr[$i] += $errr * $errr;\n    $teri[$i] += $erri * $erri;\n" <>
    "    $i++;\n}\nclose $in or die \"" <> wdir <> "/out: $!\";\n" <>
    "if($i > $maxd) {$maxd = $i;}\n\n"] &, subdirs];
  WriteString[f, "for($i = 0; $i < $maxd; $i++) {\n" <>
                 "    print \"(\", $totr[$i], \",\", $toti[$i], \") (\"," <>
		 " sqrt($terr[$i]), \",\", sqrt($teri[$i]), \")\\n\";\n}\n\n"];
  WriteString[f, "system \"rm " <> wdir <> "/out\"\n"];
  Close[f];
  Run["chmod +x " <> wdir <> "/run1"];
];
