(* tensor-reduce and evaluate two-loop vacuum integrals from global UV
   subtraction: *)
TRed[aden_, anum_, q1_, q2_] := 
Module[{IntMom, omoms, num, den, a1, a2, tden, i, T},
  IntMom = a1|a2;
  num = anum /. {q1->a1, q2->a2};
  den = aden /. {q1->a1, q2->a2};
  omoms = $ol$moms;
  $ol$moms = Alternatives @@ Join[momext, {a1,a2}];
  num = DotSimplify[num];
  (* put all numerator terms dependent on integration momenta into expression 
     with header T: *)
  num = Expand[num*T[], Dot[_,__]|T[___]];
  num = num /. Power[Dot[arga_,argb_], nA_:1] :> 
		  (T @@ Table[Dot[arga,argb], {nA}]) /; 
		  !FreeQ[Dot[arga,argb], IntMom] //.
               T[arga___] * T[argb___] :> T[arga,argb];
  num = num /. expd_Dot :> Sort[expd] /. expt_T :> Sort[expt];
  (* tensors of odd rank in the integr. momenta are zero;
     tensors of even rank in the integr. momenta evaluate to combinations of 
     metric tensors: *)
  num = num /. expt_T :> 0 /; OddQ[Length[Position[expt, IntMom]]];
  num = num  //. 
   T[argt1___, (mom1:IntMom).arga_, (mom2:IntMom).argb_, 
	       (mom3:IntMom).argc_, (mom4:IntMom).argd_,
	       (mom5:IntMom).arge_, (mom6:IntMom).argf_, argt2___] :>
    -(((arga . argf*((2 + $d)*argb . arge*argc . argd -
	   2*(argb . argd*argc . arge + argb . argc*argd . arge)) +
	 arga . arge*(-((-2 + $d*(3 + $d))*argb . argf*argc . argd) +
	   (2 + $d)*(argb . argd*argc . argf + argb . argc*argd . argf)) +
	 arga . argb*(-2*argc . argf*argd . arge - 2*argc . arge*argd . argf +
	   (2 + $d)*argc . argd*arge . argf) + arga . argd*
	  ((2 + $d)*argb . argf*argc . arge - 2*(argb . arge*argc . argf +
             argb . argc*arge . argf)) + arga . argc*
	  ((2 + $d)*argb . argf*argd . arge - 2*(argb . arge*argd . argf +
             argb . argd*arge . argf)))*T[argt1, mom3 . mom4, mom1 . mom5,
	 mom2 . mom6, argt2] +
       (arga . argf*(-((-2 + $d*(3 + $d))*argb . arge*argc . argd) +
	   (2 + $d)*(argb . argd*argc . arge + argb . argc*argd . arge)) +
	 arga . arge*((2 + $d)*argb . argf*argc . argd -
	   2*(argb . argd*argc . argf + argb . argc*argd . argf)) +
	 arga . argd*(-2*argb . argf*argc . arge + (2 + $d)*argb . arge*
            argc . argf - 2*argb . argc*arge . argf) +
	 arga . argc*(-2*argb . argf*argd . arge + (2 + $d)*argb . arge*
            argd . argf - 2*argb . argd*arge . argf) +
	 arga . argb*(-2*argc . argf*argd . arge - 2*argc . arge*argd . argf +
	   (2 + $d)*argc . argd*arge . argf))*T[argt1, mom3 . mom4, mom1 . mom6,
	 mom2 . mom5, argt2] + (arga . argf*(-2*argb . arge*argc . argd +
	   (2 + $d)*argb . argd*argc . arge - 2*argb . argc*argd . arge) +
	 arga . arge*((2 + $d)*argb . argf*argc . argd -
	   2*(argb . argd*argc . argf + argb . argc*argd . argf)) +
	 arga . argb*(-2*argc . argf*argd . arge + (2 + $d)*argc . arge*
            argd . argf - 2*argc . argd*arge . argf) +
	 arga . argd*(-((-2 + $d*(3 + $d))*argb . argf*argc . arge) +
	   (2 + $d)*(argb . arge*argc . argf + argb . argc*arge . argf)) +
	 arga . argc*((2 + $d)*argb . argf*argd . arge -
	   2*(argb . arge*argd . argf + argb . argd*arge . argf)))*
	T[argt1, mom3 . mom5, mom1 . mom4, mom2 . mom6, argt2] +
       (arga . argf*((2 + $d)*argb . arge*argc . argd - (-2 + $d*(3 + $d))*
            argb . argd*argc . arge + (2 + $d)*argb . argc*argd . arge) +
	 arga . arge*(-2*argb . argf*argc . argd + (2 + $d)*argb . argd*
            argc . argf - 2*argb . argc*argd . argf) +
	 arga . argc*(-2*argb . argf*argd . arge - 2*argb . arge*argd . argf +
	   (2 + $d)*argb . argd*arge . argf) + arga . argb*
	  (-2*argc . argf*argd . arge + (2 + $d)*argc . arge*argd . argf -
	   2*argc . argd*arge . argf) + arga . argd*
	  ((2 + $d)*argb . argf*argc . arge - 2*(argb . arge*argc . argf +
             argb . argc*arge . argf)))*T[argt1, mom3 . mom5, mom1 . mom6,
	 mom2 . mom4, argt2] + (arga . argf*((2 + $d)*argb . arge*argc . argd -
	   2*(argb . argd*argc . arge + argb . argc*argd . arge)) +
	 arga . arge*(-2*argb . argf*argc . argd + (2 + $d)*argb . argd*
            argc . argf - 2*argb . argc*argd . argf) +
	 arga . argd*((2 + $d)*argb . argf*argc . arge - (-2 + $d*(3 + $d))*
            argb . arge*argc . argf + (2 + $d)*argb . argc*arge . argf) +
	 arga . argc*(-2*argb . argf*argd . arge + (2 + $d)*argb . arge*
            argd . argf - 2*argb . argd*arge . argf) +
	 arga . argb*((2 + $d)*argc . argf*argd . arge -
	   2*(argc . arge*argd . argf + argc . argd*arge . argf)))*
	T[argt1, mom3 . mom6, mom1 . mom4, mom2 . mom5, argt2] +
       (arga . argf*(-2*argb . arge*argc . argd + (2 + $d)*argb . argd*
            argc . arge - 2*argb . argc*argd . arge) +
	 arga . arge*((2 + $d)*argb . argf*argc . argd - (-2 + $d*(3 + $d))*
            argb . argd*argc . argf + (2 + $d)*argb . argc*argd . argf) +
	 arga . argd*(-2*argb . argf*argc . arge + (2 + $d)*argb . arge*
            argc . argf - 2*argb . argc*arge . argf) +
	 arga . argc*(-2*argb . argf*argd . arge - 2*argb . arge*argd . argf +
	   (2 + $d)*argb . argd*arge . argf) + arga . argb*
	  ((2 + $d)*argc . argf*argd . arge - 2*(argc . arge*argd . argf +
             argc . argd*arge . argf)))*T[argt1, mom3 . mom6, mom1 . mom5,
	 mom2 . mom4, argt2] + (arga . argf*(-2*argb . arge*argc . argd -
	   2*argb . argd*argc . arge + (2 + $d)*argb . argc*argd . arge) +
	 arga . arge*(-2*argb . argf*argc . argd + (2 + $d)*argb . argd*
            argc . argf - 2*argb . argc*argd . argf) +
	 arga . argd*(-2*argb . argf*argc . arge + (2 + $d)*argb . arge*
            argc . argf - 2*argb . argc*arge . argf) +
	 arga . argc*((2 + $d)*argb . argf*argd . arge -
	   2*(argb . arge*argd . argf + argb . argd*arge . argf)) +
	 arga . argb*(-((-2 + $d*(3 + $d))*argc . argf*argd . arge) +
	   (2 + $d)*(argc . arge*argd . argf + argc . argd*arge . argf)))*
	T[argt1, mom4 . mom5, mom1 . mom2, mom3 . mom6, argt2] +
       (arga . argf*(-2*argb . arge*argc . argd - 2*argb . argd*argc . arge +
	   (2 + $d)*argb . argc*argd . arge) + arga . arge*
	  ((2 + $d)*argb . argf*argc . argd - 2*(argb . argd*argc . argf +
             argb . argc*argd . argf)) + arga . argd*
	  ((2 + $d)*argb . argf*argc . arge - 2*(argb . arge*argc . argf +
             argb . argc*arge . argf)) + arga . argc*
	  (-((-2 + $d*(3 + $d))*argb . argf*argd . arge) +
	   (2 + $d)*(argb . arge*argd . argf + argb . argd*arge . argf)) +
	 arga . argb*((2 + $d)*argc . argf*argd . arge -
	   2*(argc . arge*argd . argf + argc . argd*arge . argf)))*
	T[argt1, mom4 . mom5, mom1 . mom3, mom2 . mom6, argt2] +
       (arga . argf*((2 + $d)*(argb . arge*argc . argd + argb . argd*
              argc . arge) - (-2 + $d*(3 + $d))*argb . argc*argd . arge) +
	 arga . arge*(-2*argb . argf*argc . argd - 2*argb . argd*argc . argf +
	   (2 + $d)*argb . argc*argd . argf) + arga . argd*
	  (-2*argb . argf*argc . arge - 2*argb . arge*argc . argf +
	   (2 + $d)*argb . argc*arge . argf) + arga . argc*
	  ((2 + $d)*argb . argf*argd . arge - 2*(argb . arge*argd . argf +
             argb . argd*arge . argf)) + arga . argb*
	  ((2 + $d)*argc . argf*argd . arge - 2*(argc . arge*argd . argf +
             argc . argd*arge . argf)))*T[argt1, mom4 . mom5, mom1 . mom6,
	 mom2 . mom3, argt2] + (arga . argf*(-2*argb . arge*argc . argd +
	   (2 + $d)*argb . argd*argc . arge - 2*argb . argc*argd . arge) +
	 arga . arge*(-2*argb . argf*argc . argd - 2*argb . argd*argc . argf +
	   (2 + $d)*argb . argc*argd . argf) + arga . argc*
	  (-2*argb . argf*argd . arge + (2 + $d)*argb . arge*argd . argf -
	   2*argb . argd*arge . argf) + arga . argb*
	  ((2 + $d)*argc . argf*argd . arge - (-2 + $d*(3 + $d))*argc . arge*
            argd . argf + (2 + $d)*argc . argd*arge . argf) +
	 arga . argd*((2 + $d)*argb . argf*argc . arge -
	   2*(argb . arge*argc . argf + argb . argc*arge . argf)))*
	T[argt1, mom4 . mom6, mom1 . mom2, mom3 . mom5, argt2] +
       (arga . argf*((2 + $d)*argb . arge*argc . argd -
	   2*(argb . argd*argc . arge + argb . argc*argd . arge)) +
	 arga . arge*(-2*argb . argf*argc . argd - 2*argb . argd*argc . argf +
	   (2 + $d)*argb . argc*argd . argf) + arga . argd*
	  (-2*argb . argf*argc . arge + (2 + $d)*argb . arge*argc . argf -
	   2*argb . argc*arge . argf) + arga . argc*
	  ((2 + $d)*argb . argf*argd . arge - (-2 + $d*(3 + $d))*argb . arge*
            argd . argf + (2 + $d)*argb . argd*arge . argf) +
	 arga . argb*(-2*argc . argf*argd . arge + (2 + $d)*argc . arge*
            argd . argf - 2*argc . argd*arge . argf))*T[argt1, mom4 . mom6, 
	 mom1 . mom3, mom2 . mom5, argt2] + 
	 (arga . argf*(-2*argb . arge*argc . argd -
	   2*argb . argd*argc . arge + (2 + $d)*argb . argc*argd . arge) +
	 arga . arge*((2 + $d)*(argb . argf*argc . argd +
             argb . argd*argc . argf) - (-2 + $d*(3 + $d))*argb . argc*
            argd . argf) + arga . argd*(-2*argb . argf*argc . arge -
	   2*argb . arge*argc . argf + (2 + $d)*argb . argc*arge . argf) +
	 arga . argc*(-2*argb . argf*argd . arge + (2 + $d)*argb . arge*
            argd . argf - 2*argb . argd*arge . argf) +
	 arga . argb*(-2*argc . argf*argd . arge + (2 + $d)*argc . arge*
            argd . argf - 2*argc . argd*arge . argf))*T[argt1, mom4 . mom6, 
	 mom1 . mom5, mom2 . mom3, argt2] + 
	 (arga . argf*((2 + $d)*argb . arge*argc . argd -
	   2*(argb . argd*argc . arge + argb . argc*argd . arge)) +
	 arga . arge*((2 + $d)*argb . argf*argc . argd -
	   2*(argb . argd*argc . argf + argb . argc*argd . argf)) +
	 arga . argd*(-2*argb . argf*argc . arge - 2*argb . arge*argc . argf +
	   (2 + $d)*argb . argc*arge . argf) + arga . argc*
	  (-2*argb . argf*argd . arge - 2*argb . arge*argd . argf +
	   (2 + $d)*argb . argd*arge . argf) + arga . argb*
	  ((2 + $d)*(argc . argf*argd . arge + argc . arge*argd . argf) -
	   (-2 + $d*(3 + $d))*argc . argd*arge . argf))*T[argt1, mom5 . mom6, 
	 mom1 . mom2, mom3 . mom4, argt2] + 
	 (arga . argf*(-2*argb . arge*argc . argd +
	   (2 + $d)*argb . argd*argc . arge - 2*argb . argc*argd . arge) +
	 arga . arge*(-2*argb . argf*argc . argd + (2 + $d)*argb . argd*
            argc . argf - 2*argb . argc*argd . argf) +
	 arga . argd*(-2*argb . argf*argc . arge - 2*argb . arge*argc . argf +
	   (2 + $d)*argb . argc*arge . argf) + arga . argc*
	  ((2 + $d)*(argb . argf*argd . arge + argb . arge*argd . argf) -
	   (-2 + $d*(3 + $d))*argb . argd*arge . argf) +
	 arga . argb*(-2*argc . argf*argd . arge - 2*argc . arge*argd . argf +
	   (2 + $d)*argc . argd*arge . argf))*T[argt1, mom5 . mom6, mom1 . mom3,
	 mom2 . mom4, argt2] + (arga . argf*(-2*argb . arge*argc . argd -
	   2*argb . argd*argc . arge + (2 + $d)*argb . argc*argd . arge) +
	 arga . arge*(-2*argb . argf*argc . argd - 2*argb . argd*argc . argf +
	   (2 + $d)*argb . argc*argd . argf) + arga . argd*
	  ((2 + $d)*(argb . argf*argc . arge + argb . arge*argc . argf) -
	   (-2 + $d*(3 + $d))*argb . argc*arge . argf) +
	 arga . argc*(-2*argb . argf*argd . arge - 2*argb . arge*argd . argf +
	   (2 + $d)*argb . argd*arge . argf) + arga . argb*
	  (-2*argc . argf*argd . arge - 2*argc . arge*argd . argf +
	   (2 + $d)*argc . argd*arge . argf))*T[argt1, mom5 . mom6, mom1 . mom4,
	 mom2 . mom3, argt2])/($d*($d+2)*($d+4)*($d-1)*($d-2))) /;
    FreeQ[{argt1,argt2}, IntMom.(mom_/;FreeQ[mom,IntMom])] && 
    FreeQ[{arga,argb,argc,argd,arge,argf}, IntMom] //.	     
   T[argt1___, (mom1:IntMom).arga_, (mom2:IntMom).argb_, 
	       (mom3:IntMom).argc_, (mom4:IntMom).argd_, argt2___] :>
    (arga . argb*argc . argd*((1 + $d)*T[argt1, mom1 . mom2, mom3 . mom4, argt2] -
	T[argt1, mom1 . mom3, mom2 . mom4, argt2] - T[argt1, mom1 . mom4, 
	 mom2 . mom3, argt2]) + arga . argc*argb . argd*
       (-T[argt1, mom1 . mom2, mom3 . mom4, argt2] +
	(1 + $d)*T[argt1, mom1 . mom3, mom2 . mom4, argt2] - 
	T[argt1, mom1 . mom4, mom2 . mom3, argt2]) +
      arga . argd*argb . argc*(-T[argt1, mom1 . mom2, mom3 . mom4, argt2] -
	T[argt1, mom1 . mom3, mom2 . mom4, argt2] + 
	(1 + $d)*T[argt1, mom1 . mom4, mom2 . mom3, argt2]))/($d*($d+2)*($d-1)) /;
    FreeQ[{argt1,argt2}, IntMom.(mom_/;FreeQ[mom,IntMom])] && 
    FreeQ[{arga,argb,argc,argd}, IntMom] //.
   T[argt1___, (mom1:IntMom).arga_, (mom2:IntMom).argb_, argt2___] :> 
     (arga . argb*T[argt1, mom1 . mom2, argt2])/$d /;
    FreeQ[{argt1,argt2}, IntMom.(mom_/;FreeQ[mom,IntMom])] && 
    FreeQ[{arga,argb}, IntMom];
  num = num /. expd_Dot :> Sort[expd] /. expt_T :> Sort[expt];
  num = Collect[num, T[__], Simplify];
  (* add the denominator terms: *)
  tden = T @@ Map[Simplify[dn[# - (# /. Dot[_,_]->0), -# /. Dot[_,_]->0]]&, den];
  num = num /. tfun_T :> Join[tfun, tden];
  (* cancel numerators against denominator where applicable: *)
  num = num //. T[argt1___, a1.a2, argt2___, dn[(a1-a2).(a1-a2),mass_], argt3___] :>
  		1/2*(-T[argt1, argt2, argt3] - 
		     mass*T[argt1, argt2, dn[(a1-a2).(a1-a2),mass], argt3] +
		     T[argt1, a1.a1, argt2, dn[(a1-a2).(a1-a2),mass], argt3] +
		     T[argt1, a2.a2, argt2, dn[(a1-a2).(a1-a2),mass], argt3]);
  num = num //. T[argt1___, moms_, argt2___, dn[moms_,mass_], argt3___] :>
  		T[argt1, argt2, argt3] + 
		  mass*T[argt1, argt2, dn[moms,mass], argt3];
  num = num /. tfun:T[___Dot, dn[a1.a1,_].., dn[(a1-a2).(a1-a2),_]] :>
     (tfun /. a2->a1-a2 /. Dot[arga_,argb_] :> Distribute[Dot[arga,argb]] //.
      {Dot[fac_*arga_,argb_] :> fac*Dot[arga,argb] /; FreeQ[fac,IntMom],
       Dot[arga_,fac_*argb_] :> fac*Dot[arga,argb] /; FreeQ[fac,IntMom]} /.
      expd_Dot :> Sort[expd] /. T[args__] :> Distribute[T[args]] /.
      T[argt1___, fac_*argt2_, argt3___] :> fac*T[argt1, argt2, argt3] /; 
       FreeQ[fac,IntMom]);
  num = num //. T[argt1___, a1.a2, argt2___, dn[(a1-a2).(a1-a2),mass_], argt3___] :>
  		1/2*(-T[argt1, argt2, argt3] - 
		     mass*T[argt1, argt2, dn[(a1-a2).(a1-a2),mass], argt3] +
		     T[argt1, a1.a1, argt2, dn[(a1-a2).(a1-a2),mass], argt3] +
		     T[argt1, a2.a2, argt2, dn[(a1-a2).(a1-a2),mass], argt3]);
  num = num //. T[argt1___, moms_, argt2___, dn[moms_,mass_], argt3___] :>
  		T[argt1, argt2, argt3] + 
		  mass*T[argt1, argt2, dn[moms,mass], argt3];
  num = num //. T[argt1___, dn[moms_, mass1_], dn[moms_, mass2_], argt2___] :>
  	 1/(mass1-mass2) * (T[argt1, dn[moms,mass1], argt2] -
		            T[argt1, dn[moms,mass2], argt2]) /;	mass1=!=mass2;
  num = num /. T[a1.a2, dn[a1.a1,_]..., dn[a2.a2,_]...] -> 0;
  num = num /. expt_T :> Sort[expt];
  num = Collect[num, T[__], Simplify];
  (* apply reduction rules for expressions with higher powers of propagators in
     the denominator: *)
  num = num /. T[argt1:dn[a1.a1,mass1_]..., argt3:dn[(a1-a2).(a1-a2),mass3_]...,
  		 argt2:dn[a2.a2,mass2_]..] :> 
	       $tl$Tsymm[Df[k1,mass1]^Length[{argt1}]*Df[k4,mass2]^Length[{argt2}]*
	            Df[k3,mass3]^Length[{argt3}]];
  (* some two-loop integrals eventually factorize into one-loop integrals: *)
  num = num  /. $tl$Tred[Df[k1,m1_]^(n1_:1) Df[k3,m3_]^(n3_:1)] :>
    $tl$An[m1,n1] * $tl$An[m3,n3];
  (* insert analytical expressions for final basis integrals: *)
  num = num /. $tl$Tred[Df[k1,m1_] Df[k3,m3_] Df[k4,m4_]] :>
                ((m1+m3+m4)/(2*$e^2) + 
		 1/$e*(3/2*(m1+m3+m4) - xlog[m1] - xlog[m3] - xlog[m4]) +
		 f134[m1,m3,m4]) /.
	       $tl$A0[0] -> 0 /. xlog[0]->0 /. 
  	       $tl$A0[m_] :> -m*(m*Exp[-EulerGamma])^(-$e)*Gamma[$e-1];
  num = Collect[num, {f134[_], Log[__]}, Simplify];
  $ol$moms = omoms;
  num
];

lambda[m1s_,m2s_,m3s_] :=
        m1s^2 + m2s^2 + m3s^2 - 2*(m1s*m2s + m1s*m3s + m2s*m3s);

(* reduction formula for 2-loop-integral with two or three masses: *)
$tl$Tred[Df[k1,m1_]^(n1_:1) Df[k3,m3_]^(n3_:1) Df[k4,m4_]^(n4_:1)] :=
  -1 / ((n1-1)*m1*lambda[m1,m3,m4]) *
    (( (n1-1)*lambda[m1,m3,m4] + n4*(m1-m3)*(m1+m3-m4) +
       n3*(m1-m4)*(m1-m3+m4) - $d*m1*(m1-m3-m4) ) *
     $tl$Tsymm[Df[k1,m1]^(n1-1) Df[k3,m3]^n3 Df[k4,m4]^n4] +
     n4*m4*(m1+m3-m4) *
        ($tl$Tsymm[Df[k1,m1]^(n1-1) Df[k3,m3]^(n3-1) Df[k4,m4]^(n4+1)] -
         $tl$Tsymm[Df[k1,m1]^(n1-2) Df[k3,m3]^n3 Df[k4,m4]^(n4+1)]) +
     n3*m3*(m1-m3+m4) *
        ($tl$Tsymm[Df[k1,m1]^(n1-1) Df[k3,m3]^(n3+1) Df[k4,m4]^(n4-1)] -
         $tl$Tsymm[Df[k1,m1]^(n1-2) Df[k3,m3]^(n3+1) Df[k4,m4]^n4]) )   /;
  ( ((m1*lambda[m1,m3,m4])=!=0) && (n1 > 1) && (n3 > 0) && (n4 > 0) );

(* reduction formula for 2-loop-integral with one mass: *)
$tl$Tred[Df[k1,m_]^(n1_:1) Df[k3,m_]^(n3_:1) Df[k4,0]] :=
  1 / (2*(2+(n1-1)+n3-$d)*m) *
    ( ($d-n3-2*(n1-1)) * $tl$Tsymm[Df[k1,m]^(n1-1) Df[k3,m]^n3 Df[k4,0]] -
     n3 * $tl$Tsymm[Df[k1,m]^(n1-2) Df[k3,m]^(n3+1) Df[k4,0]] +
     n3 * $tl$An[m,n1-1] * $tl$An[m,n3+1] -
     2*m*n3 * $tl$An[m,n1] * $tl$An[m,n3+1] ) /; (m=!=0) && ((n1+n3) > 3);

(* sort arguments: *)
$tl$Tsymm[argt_] := Sort[{$tl$T[argt], 
                      $tl$T[argt] /. k1->KK /. k3->k1 /. KK->k3,
                      $tl$T[argt] /. k1->KK /. k4->k1 /. KK->k4,
                      $tl$T[argt] /. k3->KK /. k4->k3 /. KK->k4,
		      $tl$T[argt] /. k1->KK /. k3->k1 /. k4->k3 /. KK->k4,
		      $tl$T[argt] /. k1->KK /. k4->k1 /. k3->k4 /. KK->k3}][[1]] /.
		 $tl$T[Df[k1,m1_] Df[k3,m3_]^n3_ Df[k4,m4_]] :>
		      $tl$T[Df[k1,m3]^n3 Df[k3,m1] Df[k4,m4]] /.
		 $tl$T->$tl$Tred;
  
(* some special formulae: *)
$tl$Tred[Df[k1,m_] Df[k3,m_] Df[k4,0]] := ($d-2)/(2*($d-3)) * $tl$A0[m]^2/m;
$tl$Tred[Df[k1,m_]^2 Df[k3,m_] Df[k4,0]] := ($d-2)/4 * $tl$A0[m]^2/m^2;

(* one-loop vacuum integrals: *)
$tl$An[M_,n_] :=
        Times@@Table[($d/2-idxx)/(idxx*M), {idxx, 1, n-1}] * $tl$A0[M];
